@@ -2,10 +2,11 @@ import multi_agent_ale_py
 import os
 from pettingzoo import AECEnv
 import gym
-from gym.utils import seeding
+from gym.utils import seeding, EzPickle
 from pettingzoo.utils import agent_selector, wrappers
 from gym import spaces
 import numpy as np
+from pettingzoo.utils._parallel_env import _parallel_env_wrapper
 
 
 def base_env_wrapper_fn(raw_env_fn):
@@ -18,7 +19,11 @@ def base_env_wrapper_fn(raw_env_fn):
     return env_fn
 
 
-class BaseAtariEnv(AECEnv):
+def BaseAtariEnv(**kwargs):
+    return _parallel_env_wrapper(ParallelAtariEnv(**kwargs))
+
+
+class ParallelAtariEnv(EzPickle):
 
     metadata = {'render.modes': ['human']}
 
@@ -29,17 +34,29 @@ class BaseAtariEnv(AECEnv):
             mode_num=None,
             seed=None,
             obs_type='rgb_image',
-            frameskip=4,
             repeat_action_probability=0.25,
-            full_action_space=True):
+            full_action_space=True,
+            max_frames=100000):
         """Frameskip should be either a tuple (indicating a random range to
         choose from, with the top value exclude), or an int."""
+        EzPickle.__init__(
+            self,
+            game,
+            num_players,
+            mode_num,
+            seed,
+            obs_type,
+            repeat_action_probability,
+            full_action_space,
+            max_frames
+        )
 
         assert obs_type in ('ram', 'rgb_image', "grayscale_image"), "obs_type must  either be 'ram' or 'rgb_image' or 'grayscale_image'"
         self.obs_type = obs_type
         self.full_action_space = full_action_space
         self.num_players = num_players
         self.np_random = seeding.np_random(seed)
+        self.max_frames = max_frames
 
         multi_agent_ale_py.ALEInterface.setLoggerMode("error")
         self.ale = multi_agent_ale_py.ALEInterface()
@@ -48,7 +65,6 @@ class BaseAtariEnv(AECEnv):
             seed = seeding.create_seed(seed, max_bytes=4)
 
         self.ale.setInt(b"random_seed", seed)
-        self.ale.setInt(b"frame_skip", frameskip)
         self.ale.setFloat(b'repeat_action_probability', repeat_action_probability)
 
         pathstart = os.path.dirname(multi_agent_ale_py.__file__)
@@ -71,8 +87,12 @@ class BaseAtariEnv(AECEnv):
 
         if full_action_space:
             action_size = 18
+            action_mapping = np.arange(action_size)
         else:
-            action_size = len(self.ale.getMinimalActionSet())
+            action_mapping = self.ale.getMinimalActionSet()
+            action_size = len(action_mapping)
+
+        self.action_mapping = action_mapping
 
         if obs_type == 'ram':
             observation_space = gym.spaces.Box(low=0, high=255, dtype=np.uint8, shape=(128,))
@@ -88,27 +108,18 @@ class BaseAtariEnv(AECEnv):
         player_names = ["first", "second", "third", "fourth"]
         self.agents = [f"{player_names[n]}_0" for n in range(self.num_agents)]
 
-        self.action_spaces = {agent: gym.spaces.Discrete(action_size) for agent in self.agents}
-        self.observation_spaces = {agent: observation_space for agent in self.agents}
-        self.infos = {agent: {} for agent in self.agents}
-
-        self._agent_selector = agent_selector(self.agents)
+        self.action_spaces = [gym.spaces.Discrete(action_size)] * self.num_agents
+        self.observation_spaces = [observation_space] * self.num_agents
 
         self._screen = None
 
-    def reset(self, observe=True):
+    def reset(self):
         self.ale.reset_game()
-        self.agent_selection = self._agent_selector.reset()
+        self.frame = 0
 
-        self.rewards = {a: 0 for a in self.agents}
-        self.dones = {a: False for a in self.agents}
-        self.infos = {a: {} for a in self.agents}
+        return [self._observe()] * self.num_agents
 
-        self._actions = []
-
-        return self.observe(self.agent_selection) if observe else None
-
-    def observe(self, agent):
+    def _observe(self):
         if self.obs_type == 'ram':
             bytes = self.ale.getRAM()
             return bytes
@@ -117,26 +128,23 @@ class BaseAtariEnv(AECEnv):
         elif self.obs_type == 'grayscale_image':
             return self.ale.getScreenGrayscale()
 
-    def step(self, action, observe=True):
-        self._actions.append(action)
-        if len(self._actions) == self.num_players:
-            rewards = self.ale.act(self._actions)
-            self.rewards = {a: rew for a, rew in zip(self.agents, rewards)}
-            if self.ale.game_over():
-                self.dones = {a: True for a in self.agents}
-            else:
-                lives = self.ale.allLives()
-                # an inactive agent in ale gets a -1 life.
-                dones = [life < 0 for life in lives]
-                assert len(lives) == len(self.agents)
-                self.dones = {a: bool(d) for d, a in zip(dones, self.agents)}
-            self._actions = []
+    def step(self, actions):
+        actions = np.asarray(actions)
+        actions = self.action_mapping[actions]
+        rewards = self.ale.act(actions)
+        if self.ale.game_over() or self.frame >= self.max_frames:
+            dones = [True] * self.num_agents
+        else:
+            lives = self.ale.allLives()
+            # an inactive agent in ale gets a -1 life.
+            dones = [int(life) < 0 for life in lives]
 
-        self.agent_selection = self._agent_selector.next()
+        self.frame += 1
+        observations = [self._observe()] * self.num_agents
+        infos = [{}] * self.num_agents
+        return observations, rewards, dones, infos
 
-        return self.observe(self.agent_selection) if observe else None
-
-    def render(self, mode='human'):
+    def render(self):
         import pygame
         (screen_width, screen_height) = self.ale.getScreenDims()
         zoom_factor = 4
