@@ -67,6 +67,73 @@ class BaseWrapper(AECEnv):
         return next_obs
 
 
+class AgentIterWrapper(BaseWrapper):
+    def __init__(self, env):
+        super().__init__(env)
+        self._has_updated = False
+        # self._was_dones = {agent: False for agent in self.agents}
+
+    def reset(self, observe=True):
+        self._has_updated = True
+        self.old_observation = None
+        obs = super().reset(observe)
+        self._was_dones = {agent: done for agent, done in self.dones.items()}
+        return obs
+
+    def last(self):
+        agent = self.agent_selection
+        return self.rewards[agent], self.dones[agent], self.infos[agent]
+
+    def step(self, action, observe=True):
+        self._has_updated = True
+
+        cur_agent = self.agent_selection
+        if self.dones[cur_agent] and not self._was_dones[cur_agent]:
+            self._was_dones[cur_agent] = True
+            self.agent_selection = self.env.agent_selection
+        else:
+            super().step(action, False)
+
+        for agent in self.agents:
+            if self.dones[agent] and not self._was_dones[agent]:
+                self.agent_selection = agent
+                break
+
+        return self.observe(self.agent_selection) if observe else None
+
+    def agent_iter(self, max_agent_iter=2**63):
+        return AECIterable(self, max_agent_iter)
+
+
+class AECIterable:
+    def __init__(self, env, max_agent_iter):
+        self.env = env
+        self.max_agent_iter = max_agent_iter
+
+    def __iter__(self):
+        return AECOrderEnforcingIterator(self.env, self.max_agent_iter)
+
+
+class AECIterator:
+    def __init__(self, env, max_agent_iter):
+        self.env = env
+        self.iters_til_term = max_agent_iter
+
+    def __next__(self):
+        if self.env._was_dones[self.env.agent_selection] or self.iters_til_term <= 0:
+            raise StopIteration
+        self.iters_til_term -= 1
+        return self.env.agent_selection
+
+
+class AECOrderEnforcingIterator(AECIterator):
+    def __next__(self):
+        agent = super().__next__()
+        assert self.env._has_updated, "need to call step() or reset() in a loop over `agent_iter`!"
+        self.env._has_updated = False
+        return agent
+
+
 class TerminateIllegalWrapper(BaseWrapper):
     '''
     this wrapper terminates the game with the current player losing
@@ -185,7 +252,7 @@ class ClipOutOfBoundsWrapper(BaseWrapper):
         return super().step(action, observe)
 
 
-class OrderEnforcingWrapper(BaseWrapper):
+class OrderEnforcingWrapper(AgentIterWrapper):
     '''
     check all orders:
 
@@ -197,7 +264,6 @@ class OrderEnforcingWrapper(BaseWrapper):
     def __init__(self, env):
         self._has_reset = False
         self._has_rendered = False
-        self._has_updated = False
         super().__init__(env)
 
     def __getattr__(self, value):
@@ -208,8 +274,7 @@ class OrderEnforcingWrapper(BaseWrapper):
         if value == "agent_order":
             raise AttributeError("agent_order has been removed from the API. Please consider using agent_iter instead.")
         elif value in {"rewards", "dones", "infos", "agent_selection"}:
-            EnvLogger.error_field_before_reset(value)
-            return None
+            raise AttributeError("{} cannot be accessed before reset".format(value))
         else:
             raise AttributeError("'{}' object has no attribute '{}'".format(type(self).__name__, value))
 
@@ -230,10 +295,9 @@ class OrderEnforcingWrapper(BaseWrapper):
         self._has_reset = False
 
     def step(self, action, observe=True):
-        self._has_updated = True
         if not self._has_reset:
             EnvLogger.error_step_before_reset()
-        elif self.dones[self.agent_selection]:
+        elif self._was_dones[self.agent_selection]:
             EnvLogger.warn_step_after_done()
             self.dones = {agent: True for agent in self.dones}
             self.rewards = {agent: 0 for agent in self.rewards}
@@ -247,6 +311,5 @@ class OrderEnforcingWrapper(BaseWrapper):
         return super().observe(agent)
 
     def reset(self, observe=True):
-        self._has_updated = True
         self._has_reset = True
         return super().reset(observe)
